//
//  ViewController.swift
//  RCServo
//
//  Created by Phidgets on 2018-02-01.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    
    let ch = RCServo()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var positionLabel: UILabel!
    @IBOutlet var velocityTitle: UILabel!
    @IBOutlet var velocityLabel: UILabel!
    @IBOutlet var targetPositionLabel: UILabel!
    @IBOutlet var targetPositionSlider: UISlider!
    @IBOutlet var accelerationTitle: UILabel!
    @IBOutlet var accelerationLabel: UILabel!
    @IBOutlet var accelerationSlider: UISlider!
    @IBOutlet var velocityLimitTitle: UILabel!
    @IBOutlet var velocityLimitLabel: UILabel!
    @IBOutlet var velocityLimitSlider: UISlider!
    @IBOutlet var dataIntervalTitle: UILabel!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var speedRampingTitle: UILabel!
    @IBOutlet var speedRampingSwitch: UISwitch!
    @IBOutlet var engageButton: UIButton!
    @IBOutlet var stackView: UIStackView!
    
    //used for motor calibration
    var minPulseWidth:Double = 0
    var maxPulseWidth:Double = 0
    var positionAtMin:Double = 0
    var positionAtMax:Double = 0
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.positionChange.addHandler(positionchange_handler)
            let _ = ch.velocityChange.addHandler(velocitychange_handler)
            let _ = ch.targetPositionReached.addHandler(positionreached_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }

    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            updateGUI()
            let minPosition = try ch.getMinPosition()
            try ch.setTargetPosition(minPosition)
            DispatchQueue.main.async {
                self.targetPositionSlider.value = Float(minPosition)
                self.targetPositionLabel.text = String(format:"%.3f", Float(minPosition))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func positionchange_handler(sender: RCServo, position: Double){
        DispatchQueue.main.async {
            self.positionLabel.text = String(format:"%.3f", position)
        }
    }
    func velocitychange_handler(sender: RCServo, velocity: Double){
        DispatchQueue.main.async {
            self.velocityLabel.text = String(format:"%.3f", velocity)
        }
    }
    func positionreached_handler(sender: RCServo, position: Double){
        DispatchQueue.main.async {
            self.positionLabel.text = String(format:"%.3f", position)
        }
    }
    
    //MARK: GUI controls
    @IBAction func setTargetPosition(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setTargetPosition(Double(slider.value))
            DispatchQueue.main.async {
                self.targetPositionLabel.text = String(format:"%.3f", slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setAcceleration(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setAcceleration(Double(slider.value))
            DispatchQueue.main.async {
                self.accelerationLabel.text = String(format:"%.3f", slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setVelocityLimit(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setVelocityLimit(Double(slider.value))
            DispatchQueue.main.async {
                self.velocityLimitLabel.text = String(format:"%.3f", slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setEngaged(_ sender: Any) {
        let button = sender as! UIButton
        do{
            let engaged = try ch.getEngaged()
            try ch.setEngaged(!engaged)
            if(engaged){
                button.setTitle("Engage", for: UIControlState.normal)
            }
            else{
                button.setTitle("Disengage", for: UIControlState.normal)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setSpeedRampingState(_ sender: Any) {
        do{
            try ch.setSpeedRampingState(!(try ch.getSpeedRampingState()))
            let state = try ch.getSpeedRampingState()
            
            self.accelerationTitle.isEnabled = state
            self.accelerationLabel.isEnabled = state
            self.accelerationSlider.isEnabled = state
            
            self.velocityLimitTitle.isEnabled = state
            self.velocityLimitLabel.isEnabled = state
            self.velocityLimitSlider.isEnabled = state
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "calibrationSegue"{
            let vc:MotorCalibrationViewController = segue.destination as! MotorCalibrationViewController
            do{
                vc.minPulseWidth = try ch.getMinPulseWidth()
                vc.maxPulseWidth = try ch.getMaxPulseWidth()
                vc.positionAtMin = try ch.getMinPosition()
                vc.positionAtMax = try ch.getMaxPosition()
            } catch let err as PhidgetError {
                outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
            } catch {
                print (error)
            }
            dimBackground()
        }
    }
    
    @IBAction func calibrationSave(_ segue: UIStoryboardSegue){
        do{
            try ch.setMinPulseWidth(minPulseWidth)
            try ch.setMaxPulseWidth(maxPulseWidth)
            try ch.setMinPosition(positionAtMin)
            try ch.setMaxPosition(positionAtMax)
            updateGUI()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        dimBackground()
    }
    @IBAction func calibrationCancel(_ segue: UIStoryboardSegue){
        dimBackground()
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }
    
    func updateGUI(){
        do{
            
            var minDataInterval:Float = 0
            var maxDataInterval:Float = 0
            var dataInterval:Float = 0
            
            let deviceID = try ch.getDeviceID()
            
            let engaged = try ch.getEngaged()
            
            let speedRamping = try ch.getSpeedRampingState()
            
            let minPosition = try Float(ch.getMinPosition())
            let maxPosition = try Float(ch.getMaxPosition())
            let position = Float(0)
            
            let minAcceleration = try Float(ch.getMinAcceleration())
            let maxAcceleration = try Float(ch.getMaxAcceleration())
            let acceleration = try Float(ch.getAcceleration())
            
            let minVelocityLimit = try Float(ch.getMinVelocityLimit())
            let maxVelocityLimit = try Float(ch.getMaxVelocityLimit())
            let velocityLimit = try Float(ch.getVelocityLimit())
            
            if(deviceID == DeviceID.PN_1061 || deviceID == DeviceID.PN_1066){
                minDataInterval = try Float(ch.getMinDataInterval())
                maxDataInterval = try Float(ch.getMaxDataInterval())
                dataInterval = try Float(ch.getDataInterval())
            }
            
            DispatchQueue.main.async {
                
                self.stackView.isHidden = false
                
                self.positionLabel.text = "0.000"
                
                self.targetPositionSlider.minimumValue = minPosition
                self.targetPositionSlider.maximumValue = maxPosition
                self.targetPositionSlider.value = position
                self.targetPositionLabel.text = String(format:"%.3f", position)
                
                self.accelerationSlider.minimumValue = minAcceleration
                self.accelerationSlider.maximumValue = maxAcceleration
                self.accelerationSlider.value = acceleration
                self.accelerationLabel.text = String(format:"%.3f", acceleration)
                
                self.velocityLimitSlider.minimumValue = minVelocityLimit
                self.velocityLimitSlider.maximumValue = maxVelocityLimit
                self.velocityLimitSlider.value = velocityLimit
                self.velocityLimitLabel.text = String(format:"%.3f",velocityLimit)
                
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format:"%d ms", Int(dataInterval))
                
                self.engageButton.setTitle(engaged ? "Disengage" : "Engage", for: UIControlState.normal)
                
                self.speedRampingSwitch.isOn = speedRamping
                
                switch(deviceID){
                case DeviceID.PN_1000:
                    fallthrough
                case DeviceID.PN_1001:
                    self.isHidden(acceleration: true, velocity: false, velocityLimit: true, dataInterval: true, speedRamping: true)
                    break
                    
                case DeviceID.PN_1061:
                    fallthrough
                case DeviceID.PN_1066:
                    self.isHidden(acceleration: false, velocity: false, velocityLimit: false, dataInterval: false, speedRamping: false)
                    break
                    
                case DeviceID.PN_RCC1000:
                    self.isHidden(acceleration: false, velocity: true, velocityLimit: false, dataInterval: true, speedRamping: false)
                    break
                default:
                    break
                }
                
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func isHidden(acceleration:Bool, velocity:Bool, velocityLimit: Bool, dataInterval:Bool, speedRamping:Bool){
        DispatchQueue.main.async {
            
            self.velocityLabel.isHidden = velocity
            self.velocityTitle.isHidden = velocity
            
            self.accelerationTitle.isHidden = acceleration
            self.accelerationLabel.isHidden = acceleration
            self.accelerationSlider.isHidden = acceleration
            
            self.velocityLimitTitle.isHidden = velocityLimit
            self.velocityLimitLabel.isHidden = velocityLimit
            self.velocityLimitSlider.isHidden = velocityLimit
            
            self.dataIntervalTitle.isHidden = dataInterval
            self.dataIntervalLabel.isHidden = dataInterval
            self.dataIntervalSlider.isHidden = dataInterval
            
            self.speedRampingTitle.isHidden = speedRamping
            self.speedRampingSwitch.isHidden = speedRamping
        }
    }
}

